#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

import glob
import os

def base(fn):
    return os.path.splitext(fn)[0]

def cmd(lines):
    return " \\\n".join(lines)

# NB: *.aag and *.aig must contain a symbol table naming the primary
#     inputs and outputs, otherwise ABC and Yosys will name them
#     arbitrarily (and inconsistently with each other).

# Since ABC cannot read *.aag, read the *.aig instead
# (which would have been created by the reference aig2aig utility,
#  available from http://fmv.jku.at/aiger/)
def create_tests():
    aags = sorted(glob.glob("*.aag"))
    yss = sorted(glob.glob("*.ys"))
    for aag in aags:
        b = base(aag)

        gen_tests_makefile.generate_target(aag, cmd([
            f"$(ABC) -q \"read -c {b}.aig; write {b}_ref.v\";",
            "$(YOSYS) -qp \"",
            f"read_verilog {b}_ref.v;",
            "prep;",
            "design -stash gold;",
            f"read_aiger -clk_name clock {aag};",
            "prep;",
            "design -stash gate;",
            "design -import gold -as gold;",
            "design -import gate -as gate;",
            "miter -equiv -flatten -make_assert -make_outputs gold gate miter;",
            "sat -verify -prove-asserts -show-ports -seq 16 miter;",
            f"\" -l {aag}.log"
        ]))

    # ---- Yosys script tests ----
    for ys in yss:
        gen_tests_makefile.generate_ys_test(ys)

    gen_tests_makefile.generate_target("aigmap", cmd([
        "rm -rf gate; mkdir gate;",
        "$(YOSYS) --no-version -p \"test_cell -aigmap -w gate/ -n 1 -s 1 all\";",
        "set -o pipefail; diff --brief gold gate | tee aigmap.err;",
        "rm -f aigmap.err"
    ]))

extra = [ f"ABC ?= {gen_tests_makefile.yosys_basedir}/yosys-abc", "SHELL := /usr/bin/env bash" ]

gen_tests_makefile.generate_custom(create_tests, extra)



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

gen_tests_makefile.generate_autotest("*.v", "",
"""if grep -Eq 'expect-(wr-ports|rd-ports|rd-clk)' $@; then \\
	$(YOSYS) -f verilog -qp "proc; opt; memory -nomap; dump -outfile $(@:.v=).dmp t:\\$$mem_v2" $@; \\
	if grep -q expect-wr-ports $@; then \\
		val=$$(gawk '/expect-wr-ports/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\WR_PORTS $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected number of write ports."; exit 1; }; \\
	fi; \\
	if grep -q expect-wr-wide-continuation $@; then \\
		val=$$(gawk '/expect-wr-wide-continuation/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\WR_WIDE_CONTINUATION $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected write wide continuation."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-ports $@; then \\
		val=$$(gawk '/expect-rd-ports/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\RD_PORTS $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected number of read ports."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-clk $@; then \\
		val=$$(gawk '/expect-rd-clk/ { print $$3; }' $@); \\
		grep -Fq "connect \\\\RD_CLK $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read clock."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-en $@; then \\
		val=$$(gawk '/expect-rd-en/ { print $$3; }' $@); \\
		grep -Fq "connect \\\\RD_EN $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read enable."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-srst-sig $@; then \\
		val=$$(gawk '/expect-rd-srst-sig/ { print $$3; }' $@); \\
		grep -Fq "connect \\\\RD_SRST $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read sync reset."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-srst-val $@; then \\
		val=$$(gawk '/expect-rd-srst-val/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\RD_SRST_VALUE $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read sync reset value."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-arst-sig $@; then \\
		val=$$(gawk '/expect-rd-arst-sig/ { print $$3; }' $@); \\
		grep -Fq "connect \\\\RD_ARST $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read async reset."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-arst-val $@; then \\
		val=$$(gawk '/expect-rd-arst-val/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\RD_ARST_VALUE $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read async reset value."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-init-val $@; then \\
		val=$$(gawk '/expect-rd-init-val/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\RD_INIT_VALUE $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read init value."; exit 1; }; \\
	fi; \\
	if grep -q expect-rd-wide-continuation $@; then \\
		val=$$(gawk '/expect-rd-wide-continuation/ { print $$3; }' $@); \\
		grep -Fq "parameter \\\\RD_WIDE_CONTINUATION $$val" $(@:.v=).dmp || { echo " ERROR: Unexpected read wide continuation."; exit 1; }; \\
	fi; \\
	if grep -q expect-no-rd-clk $@; then \\
		grep -Fq "connect \\\\RD_CLK 1'x" $(@:.v=).dmp || { echo " ERROR: Expected no read clock."; exit 1; }; \\
	fi; \\
fi""")



#!/usr/bin/env python3

import glob
import os
import sys
sys.path.append("..")

import gen_tests_makefile

def lib_tests():
    for lib in sorted(glob.glob("*.lib")):
        base = os.path.splitext(lib)[0]

        gen_tests_makefile.generate_cmd_test(lib, [
            f'$(YOSYS) -p "read_verilog small.v; synth -top small; dfflibmap -info -liberty {lib}" -ql {base}.log',

            f'$(YOSYS_FILTERLIB) - {lib} > {lib}.filtered',
            f'$(YOSYS_FILTERLIB) -verilogsim {lib} > {lib}.verilogsim',

            f'diff {lib}.filtered {lib}.filtered.ok',
            f'diff {lib}.verilogsim {lib}.verilogsim.ok',

            f'if [ -e {base}.log.ok ]; then '
            f'$(YOSYS) -p "dfflibmap -info -liberty {lib}" -TqqQl {base}.log; '
            f'diff {base}.log {base}.log.ok; '
            f'fi',
        ])


def ys_tests():
    for ys in sorted(glob.glob("*.ys")):
        gen_tests_makefile.generate_ys_test(ys)

def main():
    def callback():
        lib_tests()
        ys_tests()

    gen_tests_makefile.generate_custom(callback,
        [f"YOSYS_FILTERLIB ?= {gen_tests_makefile.yosys_basedir}/yosys-filterlib"])


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

def create_tests():
    setup = "mkdir -p temp && cp content1.dat temp/content2.dat"

    gen_tests_makefile.generate_cmd_test("parent_content1", [
        f"{setup}",
        'cd .. && $(YOSYS_ABS) -qp "read_verilog -defer memfile/memory.v; '
        'chparam -set MEMFILE \\"content1.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("parent_content2_temp", [
        f"{setup}",
        'cd .. && $(YOSYS_ABS) -qp "read_verilog -defer memfile/memory.v; '
        'chparam -set MEMFILE \\"temp/content2.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("parent_content2_full", [
        f"{setup}",
        'cd .. && $(YOSYS_ABS) -qp "read_verilog -defer memfile/memory.v; '
        'chparam -set MEMFILE \\"memfile/temp/content2.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("same_content1", [
        f"{setup}",
        '$(YOSYS) -qp "read_verilog -defer memory.v; '
        'chparam -set MEMFILE \\"content1.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("same_content2", [
        f"{setup}",
        '$(YOSYS) -qp "read_verilog -defer memory.v; '
        'chparam -set MEMFILE \\"temp/content2.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("child_content1", [
        f"{setup}",
        'cd temp && ../$(YOSYS) -qp "read_verilog -defer ../memory.v; '
        'chparam -set MEMFILE \\"content1.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("child_content2_temp", [
        f"{setup}",
        'cd temp && ../$(YOSYS) -qp "read_verilog -defer ../memory.v; '
        'chparam -set MEMFILE \\"temp/content2.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("child_content2_direct", [
        f"{setup}",
        'cd temp && ../$(YOSYS) -qp "read_verilog -defer ../memory.v; '
        'chparam -set MEMFILE \\"temp/content2.dat\\" memory"'
    ])

    gen_tests_makefile.generate_cmd_test("fail_empty_filename",
        '! $(YOSYS) -qp "read_verilog memory.v"')

    gen_tests_makefile.generate_cmd_test("fail_missing_file", [
        '! $(YOSYS) -qp "read_verilog -defer memory.v; '
        'chparam -set MEMFILE \\"content3.dat\\" memory"'
    ])

extra = ["YOSYS_ABS := $(abspath $(YOSYS))"]
gen_tests_makefile.generate_custom(create_tests, extra)



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

import subprocess
from pathlib import Path

print("Generate FST for sim models")

for name in Path("tb").rglob("tb*.v"):
    test_name = name.stem 
    print(f"Test {test_name}")

    verilog_name = f"{test_name[3:]}.v"

    out_file = Path("tb") / f"{test_name}.out"

    subprocess.run(
        ["iverilog", "-o", str(out_file), str(name), verilog_name],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
        check=True
    )

    subprocess.run(
        [str(out_file), "-fst"],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
        check=True
    )

gen_tests_makefile.generate(["--yosys-scripts", "--bash", "--yosys-args", "-w 'Yosys has only limited support for tri-state logic at the moment.'" ])



#!/usr/bin/env python3

import glob
import os
import sys
import argparse
from contextlib import contextmanager

yosys_basedir = os.path.relpath(os.path.join(os.path.dirname(__file__), ".."))
common_mk = os.path.relpath(os.path.join(os.path.dirname(__file__), "common.mk"))

def _cwd_base():
    return os.path.basename(os.getcwd())

def generate_target(name, command, deps = None):
    #target = f"{_cwd_base()}-{name}"
    target = f"{name}"
    print(f"all: {target}")
    print(f".PHONY: {target}")
    print(f"{target}_cmd={command}")
    if deps:
        print(f"{target}: {' '.join(deps)}")
    else:
        print(f"{target}:")

    print(f"\t@$(call run_test,{target}, $({target}_cmd))")

def generate_ys_test(ys_file, yosys_args="", commands=""):
    cmd = f'$(YOSYS) -ql {ys_file}.err {yosys_args} {ys_file} && mv {ys_file}.err {ys_file}.log'
    if commands:
        cmd += f"; \\\n{commands}"
    generate_target(ys_file, cmd)

def generate_tcl_test(tcl_file, yosys_args="", commands=""):
    cmd = f'$(YOSYS) -ql {tcl_file}.err {yosys_args} {tcl_file} && mv {tcl_file}.err {tcl_file}.log'
    if commands:
        cmd += f"; \\\n{commands}"
    generate_target(tcl_file, cmd)

def generate_sv_test(sv_file, yosys_args="", commands=""):
    base = os.path.splitext(sv_file)[0]
    if not os.path.exists(base + ".ys"):
        yosys_cmd = '-p "prep -top top; async2sync; sat -enable_undef -verify -prove-asserts"'
        cmd = f'$(YOSYS) -ql {sv_file}.err {yosys_cmd} {yosys_args} {sv_file} && mv {sv_file}.err {sv_file}.log'
        if commands:
            cmd += f"; \\\n{commands}"
        generate_target(sv_file, cmd)

def generate_bash_test(sh_file, commands=""):
    cmd = f"bash -v {sh_file} >{sh_file}.err 2>&1 && mv {sh_file}.err {sh_file}.log"
    if commands:
        cmd += f"; \\\n{commands}"
    generate_target(sh_file, cmd)

def unpack_cmd(cmd):
    if isinstance(cmd, str):
        return cmd
    if isinstance(cmd, (list, tuple)):
        return " && \\\n".join(cmd)
    raise TypeError("cmd must be a string or a list/tuple of strings")

def generate_cmd_test(test_name, cmd, yosys_args="", deps = None):
    generate_target(test_name, unpack_cmd(cmd), deps)

def generate_tests(argv, cmds):
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("-y", "--yosys-scripts", action="store_true")
    parser.add_argument("-t", "--tcl-scripts", action="store_true")
    parser.add_argument("-s", "--prove-sv", action="store_true")
    parser.add_argument("-b", "--bash", action="store_true")
    parser.add_argument("-a", "--yosys-args", default="")

    args = parser.parse_args(argv)

    if not (args.yosys_scripts or args.tcl_scripts or args.prove_sv or args.bash):
        raise RuntimeError("No file types selected")

    if args.yosys_scripts:
        for f in sorted(glob.glob("*.ys")):
            generate_ys_test(f, args.yosys_args, cmds)

    if args.tcl_scripts:
        for f in sorted(glob.glob("*.tcl")):
            generate_tcl_test(f, args.yosys_args, cmds)

    if args.prove_sv:
        for f in sorted(glob.glob("*.sv")):
            generate_sv_test(f, args.yosys_args, cmds)

    if args.bash:
        for f in sorted(glob.glob("*.sh")):
            if f != "run-test.sh":
                generate_bash_test(f, cmds)

def print_header(extra=None):
    print(f"include {common_mk}")
    print(f"YOSYS ?= {yosys_basedir}/yosys")
    print("")
    print("export YOSYS_MAX_THREADS := 4")
    if extra:
        for line in extra:
            print(line)
    print("")
    print(".PHONY: all")
    print("all:")

@contextmanager
def redirect_stdout(new_target):
    old_target, sys.stdout = sys.stdout, new_target
    try:
        yield new_target
    finally:
        sys.stdout = old_target

def generate(argv, extra=None, cmds=""):
    with open("Makefile", "w") as f:
        with redirect_stdout(f):
            print_header(extra)
            generate_tests(argv, cmds)

def generate_custom(callback, extra=None):
    with open("Makefile", "w") as f:
        with redirect_stdout(f):
            print_header(extra)
            callback()

def generate_autotest_file(test_file, commands):
    cmd = f"../tools/autotest.sh -G -j ${{SEEDOPT}} ${{EXTRA_FLAGS}} {test_file}; \\\n{commands}"
    generate_target(test_file, cmd)

def generate_autotest(pattern, extra_flags, cmds=""):
    with open("Makefile", "w") as f:
        with redirect_stdout(f):
            print_header([ f"EXTRA_FLAGS = {extra_flags}",
                           "SEED ?=",
                           "ifneq ($(strip $(SEED)),)",
                           "    SEEDOPT=-S$(SEED)",
                           "endif",
                        ])
            for fn in sorted(glob.glob(pattern)):
                generate_autotest_file(fn, cmds)



#!/usr/bin/env python3

from pathlib import Path
import sys
sys.path.append("..")

import gen_tests_makefile

techlibs_dir = Path("../../techlibs")

# Architecture-specific defines
defines = {
    "ice40": ["ICE40_HX", "ICE40_LP", "ICE40_U"]
}

def archs():
    # Loop over architectures
    for arch in techlibs_dir.iterdir():
        if not arch.is_dir():
            continue
        arch_name = arch.name

        for path in arch.rglob("cells_sim.v"):
            rel_parts = path.relative_to(techlibs_dir).parts
            target_base = "_".join(rel_parts[-len(rel_parts):]).replace(".v", "")
            path_str = str(path)
            if arch_name in defines:
                for defn in defines[arch_name]:
                    target_name = f"{target_base}_{defn}"
                    cmd = f"iverilog -t null -I{arch} -D{defn} -DNO_ICE40_DEFAULT_ASSIGNMENTS {path_str}"
                    gen_tests_makefile.generate_target(target_name, cmd)
            else:
                target_name = f"{target_base}"
                cmd = f"iverilog -t null -I{arch} -g2005-sv {path_str}"
                gen_tests_makefile.generate_target(target_name, cmd)

def common():
    for path in ["../../techlibs/common/simcells.v", "../../techlibs/common/simlib.v"]:
        path_obj = Path(path)
        target_name = path_obj.stem
        cmd = f"iverilog -t null {path}"
        gen_tests_makefile.generate_target(target_name, cmd)

def main():
    def callback():
        archs()
        common()

    gen_tests_makefile.generate_custom(callback)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

def initial_display():
    gen_tests_makefile.generate_cmd_test("initial_display", [
        f"$(YOSYS) -p \"read_verilog initial_display.v\" | awk '/<<<BEGIN>>>/,/<<<END>>>/ {{print $$0}}' >yosys-initial_display.log 2>&1",
        "iverilog -o iverilog-initial_display initial_display.v",
        "./iverilog-initial_display >iverilog-initial_display.log",
        "diff yosys-initial_display.log iverilog-initial_display.log",
    ])


def always_display():
    cases = [
        ("clk", "-DEVENT_CLK"),
        ("clk_rst", "-DEVENT_CLK_RST"),
        ("star", "-DEVENT_STAR"),
        ("clk_en", "-DEVENT_CLK -DCOND_EN"),
        ("clk_rst_en", "-DEVENT_CLK_RST -DCOND_EN"),
        ("star_en", "-DEVENT_STAR -DCOND_EN"),
    ]

    for name, defs in cases:
        gen_tests_makefile.generate_cmd_test(f"always_display_{name}", [
            f"$(YOSYS) -p \"read_verilog {defs} always_display.v; proc; opt_expr -mux_bool; clean\" -o yosys-always_display-{name}-1.v",
            f"$(YOSYS) -p \"read_verilog yosys-always_display-{name}-1.v; proc; opt_expr -mux_bool; clean\" -o yosys-always_display-{name}-2.v",
            f"diff yosys-always_display-{name}-1.v yosys-always_display-{name}-2.v",
        ])


def roundtrip():
    cases = [
        ("dec_unsigned", '-DBASE_DEC -DSIGN=""'),
        ("dec_signed", '-DBASE_DEC -DSIGN="signed"'),
        ("hex_unsigned", '-DBASE_HEX -DSIGN=""'),
        ("hex_signed", '-DBASE_HEX -DSIGN="signed"'),
        ("oct_unsigned", '-DBASE_HEX -DSIGN=""'),
        ("oct_signed", '-DBASE_HEX -DSIGN="signed"'),
        ("bin_unsigned", '-DBASE_HEX -DSIGN=""'),
        ("bin_signed", '-DBASE_HEX -DSIGN="signed"'),
    ]

    for name, defs in cases:
        gen_tests_makefile.generate_cmd_test(f"roundtrip_{name}", [
            f"$(YOSYS) -p \"read_verilog {defs} roundtrip.v; proc; clean\" -o yosys-roundtrip-{name}-1.v",
            f"$(YOSYS) -p \"read_verilog yosys-roundtrip-{name}-1.v; proc; clean\" -o yosys-roundtrip-{name}-2.v",
            f"diff yosys-roundtrip-{name}-1.v yosys-roundtrip-{name}-2.v",

            f"iverilog {defs} -o iverilog-roundtrip-{name} roundtrip.v roundtrip_tb.v",
            f"./iverilog-roundtrip-{name} >iverilog-roundtrip-{name}.log",

            f"iverilog {defs} -o iverilog-roundtrip-{name}-1 yosys-roundtrip-{name}-1.v roundtrip_tb.v",
            f"./iverilog-roundtrip-{name}-1 >iverilog-roundtrip-{name}-1.log",

            f"iverilog {defs} -o iverilog-roundtrip-{name}-2 yosys-roundtrip-{name}-2.v roundtrip_tb.v",
            f"./iverilog-roundtrip-{name}-2 >iverilog-roundtrip-{name}-2.log",

            f"diff iverilog-roundtrip-{name}.log iverilog-roundtrip-{name}-1.log",
            f"diff iverilog-roundtrip-{name}-1.log iverilog-roundtrip-{name}-2.log",
        ])


def cxxrtl():
    cases = ["always_full", "always_comb"]

    for name in cases:
        gen_tests_makefile.generate_cmd_test(f"cxxrtl_{name}", [
            f"$(YOSYS) -p \"read_verilog {name}.v; proc; clean; write_cxxrtl -print-output std::cerr yosys-{name}.cc\"",
            f"$${{CXX:-g++}} -std=c++11 -o yosys-{name} -I../../backends/cxxrtl/runtime {name}_tb.cc -lstdc++",
            f"./yosys-{name} 2>yosys-{name}.log",

            f"iverilog -o iverilog-{name} {name}.v {name}_tb.v",
            f"./iverilog-{name} | grep -v \"$finish called\" >iverilog-{name}.log",

            f"diff iverilog-{name}.log yosys-{name}.log",
        ])


def extra():
    gen_tests_makefile.generate_cmd_test("always_full_equiv", [
        "$(YOSYS) -p \"read_verilog always_full.v; prep; clean\" -o yosys-always_full-1.v",
        "iverilog -o iverilog-always_full-1 yosys-always_full-1.v always_full_tb.v",
        "./iverilog-always_full-1 > tmp.log",
        "grep -v '\\$finish called' tmp.log > iverilog-always_full-1.log",
        "diff iverilog-always_full.log iverilog-always_full-1.log",
    ], deps=["cxxrtl_always_full"])

    gen_tests_makefile.generate_cmd_test("display_lm", [
        "$(YOSYS) -p \"read_verilog display_lm.v\" >yosys-display_lm.log 2>&1",
        "$(YOSYS) -p \"read_verilog display_lm.v; write_cxxrtl yosys-display_lm.cc\"",
        f"$${{CXX:-g++}} -std=c++11 -o yosys-display_lm_cc -I../../backends/cxxrtl/runtime display_lm_tb.cc -lstdc++",
        "./yosys-display_lm_cc >yosys-display_lm_cc.log",
        "grep \"^%l: \\\\\\bot\\$$\" \"yosys-display_lm.log\"",
        "grep \"^%m: \\\\\\bot\\$$\" \"yosys-display_lm.log\"",
        "grep \"^%l: \\\\\\bot\\$$\" \"yosys-display_lm_cc.log\"",
        "grep \"^%m: \\\\\\bot\\$$\" \"yosys-display_lm_cc.log\"",
    ])


def main():
    def callback():
        initial_display()
        always_display()
        roundtrip()
        cxxrtl()
        extra()

    gen_tests_makefile.generate_custom(callback)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from pathlib import Path
import glob
import sys
sys.path.append("..")

import gen_tests_makefile

runone_tests = [
    "svinterface1",
    #"svinterface_at_top"
]

def run_one():
    for testname in runone_tests:
        cmd_lines = [
            f'$(YOSYS) -p "read_verilog -sv {testname}.sv ; hierarchy -check -top TopModule ; synth ; write_verilog {testname}_syn.v" >> {testname}.log_stdout 2>> {testname}.log_stderr',
            f'$(YOSYS) -p "read_verilog -sv {testname}_ref.v ; hierarchy -check -top TopModule ; synth ; write_verilog {testname}_ref_syn.v" >> {testname}.log_stdout 2>> {testname}.log_stderr',
            f'rm -f a.out reference_result.txt dut_result.txt',
            f'iverilog -g2012 {testname}_syn.v',
            f'iverilog -g2012 {testname}_ref_syn.v',
            f'iverilog -g2012 {testname}_tb.v {testname}_ref_syn.v',
            f'./a.out',
            f'mv output.txt reference_result.txt',
            f'iverilog -g2012 {testname}_tb_wrapper.v {testname}_syn.v' if testname=="svinterface_at_top" else
            f'iverilog -g2012 {testname}_tb.v {testname}_syn.v',
            f'./a.out',
            f'mv output.txt dut_result.txt',
            f'diff reference_result.txt dut_result.txt > {testname}.diff',
        ]
        gen_tests_makefile.generate_cmd_test(testname, cmd_lines)

def run_simple():
    for f in sorted(glob.glob("*.ys")):
        gen_tests_makefile.generate_ys_test(f)

def main():
    def callback():
        run_one()
        run_simple()

    gen_tests_makefile.generate_custom(callback)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

gen_tests_makefile.generate(["--yosys-scripts", "--tcl-scripts", "--bash", "--yosys-args", "-e 'select out of bounds'"])



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

def callback():
    gen_tests_makefile.generate_cmd_test("functional",f'pytest -v -m "not smt and not rkt" . "$$@"')

gen_tests_makefile.generate_custom(callback)



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile
import glob

def create_tests():
    yss = sorted(glob.glob("*.ys"))
    for ys in yss:
        gen_tests_makefile.generate_ys_test(ys)

    cmd = [ "python3 frontend.py unix-socket frontend.sock" ]
    gen_tests_makefile.generate_cmd_test("frontend.py", cmd)

gen_tests_makefile.generate_custom(create_tests)



#!/usr/bin/env python3

import sys
sys.path.append("..")

import shutil

# ----------------------
# Check if iverilog is installed
# ----------------------
if shutil.which("iverilog") is None:
    print("Error: Icarus Verilog 'iverilog' not found.", file=sys.stderr)
    sys.exit(1)

import gen_tests_makefile

gen_tests_makefile.generate_autotest("*.*v", "")



#!/usr/bin/env python3

import sys
sys.path.append("../..")

import gen_tests_makefile
import mem_gen

gen_tests_makefile.generate(["--yosys-scripts", "--bash", "--yosys-args", "-w 'Yosys has only limited support for tri-state logic at the moment.'" ])



#!/usr/bin/env python3

import sys
sys.path.append("..")

import shutil
import os
from pathlib import Path

# ----------------------
# Check if iverilog is installed
# ----------------------
if shutil.which("iverilog") is None:
    print("Error: Icarus Verilog 'iverilog' not found.", file=sys.stderr)
    sys.exit(1)

src_dir = Path("../simple")
# ----------------------
# Copy all files from ../simple to current directory
# ----------------------
for file in src_dir.glob("*.v"):
    shutil.copy(file, os.path.join(".", file.name))

for file in src_dir.glob("*.sv"):
    shutil.copy(file, os.path.join(".", file.name))

# bug 2675
bug_file = os.path.join(".", "specify.v")
if os.path.exists(bug_file):
    os.remove(bug_file)

import gen_tests_makefile

gen_tests_makefile.generate_autotest("*.*v", "-f \"verilog -noblackbox -specify\" -n 300 -p '\
    read_verilog -icells -lib +/abc9_model.v; \
    hierarchy; \
    synth -run coarse; \
    opt -full; \
    techmap; \
    abc9 -lut 4; \
    clean; \
    check -assert * abc9_test037 %d; \
    select -assert-none t:?_NOT_ t:?_AND_ %%; \
    setattr -mod -unset blackbox -unset whitebox =*'")



#!/usr/bin/env python3

import sys
sys.path.append("..")

import gen_tests_makefile

def run_subtest(name):
    gen_tests_makefile.generate_cmd_test(f"cxxrtl_{name}", [
        f"$${{CXX:-g++}} -std=c++11 -O2 -o cxxrtl-test-{name} -I../../backends/cxxrtl/runtime test_{name}.cc -lstdc++",
        f"./cxxrtl-test-{name}",
    ])

def compile_only():
    gen_tests_makefile.generate_cmd_test("cxxrtl_unconnected_output", [
        '$(YOSYS) -p "read_verilog test_unconnected_output.v; select =*; proc; clean; write_cxxrtl cxxrtl-test-unconnected_output.cc"',
        f'$${{CXX:-g++}} -std=c++11 -c -o cxxrtl-test-unconnected_output -I../../backends/cxxrtl/runtime cxxrtl-test-unconnected_output.cc',
    ])

def main():
    def callback():
        run_subtest("value")
        run_subtest("value_fuzz")
        compile_only()

    gen_tests_makefile.generate_custom(callback)

if __name__ == "__main__":
    main()
